﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace GUIDesigner
{
	public partial class Form1 : Form
	{
		private List<MyPictureBox> pictures;
		private MyPictureBox activePicture;

		private bool canMoveImage;
		private int offsetX;
		private int offsetY;

        private bool textChangedEnabled;
		

		[DllImport("Kernel32.dll")]
		static extern Boolean AllocConsole();


		public Form1()
		{
			if (!AllocConsole())
			{
				MessageBox.Show("Failed");
			}
			InitializeComponent();

			this.basicScreen.Size = new Size(800, 600);
			this.basicScreen.Location = new Point(20, 20);

			this.pictures = new List<MyPictureBox>();

			this.canMoveImage = false;

			this.activePicture = null;
            this.textChangedEnabled = true;
		}


		private void Picture_OnMouseDown(object sender, MouseEventArgs e)
		{
			this.canMoveImage = true;
		
			this.offsetX = e.X;
			this.offsetY = e.Y;

			this.activePicture = (MyPictureBox)sender;
			this.imgPreview.Image = this.activePicture.Image;
			
			this.grpBoxDimension.Visible = true;
			this.grpBoxImage.Visible = true;
			this.grpBoxPos.Visible = true;
			
			this.txtBoxPath.Text = this.activePicture.Path;
			this.imgPreview.BackColor = this.activePicture.BackColor;

            this.CalcPictureBoxSize(this.activePicture);


			this.UpdatePositionInfo();

		}

		private void Picture_OnMouseMove(object sender, MouseEventArgs e)
		{
			
			if (this.canMoveImage == false)
			{
				return;
			}

			PictureBox p = (PictureBox)sender;

			Point oldPos = p.Location;			
			Point newPos = new Point(0, 0);

			newPos.X = oldPos.X + e.X - offsetX;
			newPos.Y = oldPos.Y + e.Y - offsetY;

			p.Location = newPos;

			this.UpdatePositionInfo();
		}

		private void Picture_OnMouseUp(object sender, MouseEventArgs e)
		{
			this.canMoveImage = false;
		}

		private void addElementItem_Click(object sender, EventArgs e)
		{
			//Panel parent = (Panel)sender;

			ToolStripItem item = (sender as ToolStripItem);
			ContextMenuStrip owner = null;
			if (item != null)
			{
				owner = (item.Owner as ContextMenuStrip);				
			}

			if (owner == null)
			{
				return;
			}

			Random random = new Random();
			int randomNumber = random.Next(0, 100);


			MyPictureBox p = new MyPictureBox();
			//p.Location = new Point(400, 10);

			Rectangle screenRectangle = RectangleToScreen(this.ClientRectangle);
			int titleHeight = screenRectangle.Top - this.Top;

			Point point = owner.Bounds.Location;
			point.X -= owner.SourceControl.Bounds.Location.X;
			point.Y -= owner.SourceControl.Bounds.Location.Y;
			point.X -= this.Bounds.Location.X;
			point.Y -= this.Bounds.Location.Y;
			point.Y -= titleHeight;

			p.Location = point;
            p.WidthPercent = 0.1f;
            p.HeightPercent = 0.1f;
            p.WidthOffset = 0;
            p.HeightOffset = 0;

			p.SizeMode = PictureBoxSizeMode.StretchImage;
			p.BackColor = Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255));

			p.Parent = owner.SourceControl;

			p.MouseDown += this.Picture_OnMouseDown;
			p.MouseMove += this.Picture_OnMouseMove;
			p.MouseUp += this.Picture_OnMouseUp;
			p.ContextMenuStrip = this.rightClickMenu;


			p.BringToFront();
			this.CalcPictureBoxSize(p);
            
			this.pictures.Add(p);

		}

		private void removeElementToolStripMenuItem_Click(object sender, EventArgs e)
		{
			MyPictureBox owner = this.GetToolstripPictureBox(sender);			
			if (owner == null)
			{
				return;
			}

			owner.Parent = null;

			this.pictures.Remove(owner);

			Control.ControlCollection o = owner.Controls;

			//Console.WriteLine("x");
		}

		private void bringToFrontToolStripMenuItem_Click(object sender, EventArgs e)
		{
			MyPictureBox owner = this.GetToolstripPictureBox(sender);			
			if (owner == null)
			{
				return;
			}

			owner.BringToFront();
		}

		private void sendToBackToolStripMenuItem_Click(object sender, EventArgs e)
		{
			MyPictureBox owner = this.GetToolstripPictureBox(sender);			
			if (owner == null)
			{
				return;
			}

			owner.SendToBack();
		}

		private void txtBox_TextChanged(object sender, EventArgs e)
		{
            if (this.textChangedEnabled == false)
            {
                return;
            }

			if (this.activePicture == null)
			{
				return;
			}

			TextBox txtBox = sender as TextBox;
			if (txtBox == null)
			{
				return;
			}

            String txt = txtBox.Text;
            txt = txt.Replace(".", ",");

            if (txt.EndsWith(","))
            {
                return;
            }

			float res = -1;
			if (float.TryParse(txt, out res) == false)
			{
				return;
			}

			if (res <= 0)
			{
				return;
			}

			

			if (txtBox == this.txtBoxWidthPercent)
			{
				this.activePicture.WidthPercent = res;
			}
            else if (txtBox == this.txtBoxHeightPercent)
			{
				this.activePicture.HeightPercent = res;
			}
			else if (txtBox == this.txtBoxOffsetWidthPx)
			{
				this.activePicture.WidthOffset = (int)res;
			}
			else if (txtBox == this.txtBoxOffsetHeightPx)
			{
				this.activePicture.HeightOffset = (int)res;
			}
            else if (txtBox == this.txtBoxRealWidth)
            {
                this.activePicture.Size = new Size((int)res, this.activePicture.Size.Height);
                this.UpdatePercents(this.activePicture);
            }
            else if (txtBox == this.txtBoxRealHeight)
            {
                this.activePicture.Size = new Size(this.activePicture.Size.Width, (int)res);
                this.UpdatePercents(this.activePicture);
            }

			this.CalcPictureBoxSize(this.activePicture);

			
			if (this.chckBoxKeepAR.Checked)
			{
				this.ChangeAR();
			}
		}

		private void basicScreen_MouseDown(object sender, MouseEventArgs e)
		{
			this.activePicture = null;
			this.grpBoxDimension.Visible = false;
			this.grpBoxImage.Visible = false;
			this.grpBoxPos.Visible = false;
		}

		private void txtBoxPath_Click(object sender, EventArgs e)
		{
			DialogResult result = this.openFileDialog1.ShowDialog();
			if (result != DialogResult.OK) // Test result.
			{
				return;
			}

			String file = this.openFileDialog1.FileName;
			this.txtBoxPath.Text = file;

			this.activePicture.Image = Image.FromFile(file);
			this.activePicture.Path = file;
			this.activePicture.BackColor = Color.Transparent;
			this.imgPreview.Image = this.activePicture.Image;
            
		}

		private void chckBoxKeepAR_CheckedChanged(object sender, EventArgs e)
		{
			this.ChangeAR();

			this.txtBoxHeightPercent.Enabled = true;
			this.txtBoxOffsetHeightPx.Enabled = true;
            this.txtBoxRealHeight.Enabled = true;

			if (this.chckBoxKeepAR.Checked)
			{
				this.txtBoxHeightPercent.Enabled = false;
				this.txtBoxOffsetHeightPx.Enabled = false;
                this.txtBoxRealHeight.Enabled = false;
			}
		}

		MyPictureBox GetToolstripPictureBox(object sender)
		{
			ToolStripItem item = (sender as ToolStripItem);
			MyPictureBox owner = null;
			if (item != null)
			{
				ContextMenuStrip tmp = (item.Owner as ContextMenuStrip);
				if (tmp != null)
				{
					owner = (tmp.SourceControl as MyPictureBox);
				}
			}

			if (owner == null)
			{
				return null;
			}

			return owner;
		}

		void ChangeAR()
		{
			if (this.activePicture.Image == null)
			{
				return;
			}

            this.textChangedEnabled = false;
			
            this.CalcPictureBoxSize(this.activePicture);

			double ar = (double)this.activePicture.Image.Size.Height / (double)this.activePicture.Image.Size.Width;

			double tmp = (double)this.activePicture.Size.Width * ar;
			tmp = tmp / (double)this.activePicture.Parent.Height;


			this.activePicture.HeightPercent = tmp;
            this.activePicture.HeightOffset = 0;

			
            this.txtBoxWidthPercent.Text = "" + Math.Round(this.activePicture.WidthPercent, 2);
            this.txtBoxHeightPercent.Text = "" + Math.Round(this.activePicture.HeightPercent, 2);

            this.txtBoxOffsetWidthPx.Text = "" + this.activePicture.WidthOffset;
            this.txtBoxOffsetHeightPx.Text = "" + this.activePicture.HeightOffset;

            this.CalcPictureBoxSize(this.activePicture);

            this.textChangedEnabled = true;
		}

		void UpdatePositionInfo()
		{
			int parentW = this.activePicture.Parent.Size.Width;
			int parentH = this.activePicture.Parent.Size.Height;

			//Top Left
			double x = this.activePicture.Location.X / (double)parentW;
			double y = this.activePicture.Location.Y / (double)parentH;

			this.activePicture.TL_X = x;
			this.activePicture.TL_Y = y;

			this.lblPosXTL.Text = "" + Math.Round(x * 100, 2) + " %";
			this.lblPosYTL.Text = "" + Math.Round(y * 100, 2) + " %";

			//Top Right
			x = 1.0 - (this.activePicture.Location.X + this.activePicture.Size.Width) / (double)parentW;
			y = this.activePicture.Location.Y / (double)parentH;

			this.lblPosXTR.Text = "" + Math.Round(x * 100, 2) + " %";
			this.lblPosYTR.Text = "" + Math.Round(y * 100, 2) + " %";

			//Bottom left
			x = (this.activePicture.Location.X) / (double)parentW;
			y = 1.0 - (this.activePicture.Location.Y + this.activePicture.Size.Height) / (double)parentH;

			this.lblPosXBL.Text = "" + Math.Round(x * 100, 2) + " %";
			this.lblPosYBL.Text = "" + Math.Round(y * 100, 2) + " %";

			//Bottom right
			x = 1.0 - (this.activePicture.Location.X + this.activePicture.Size.Width) / (double)parentW;
			y = 1.0 - (this.activePicture.Location.Y + this.activePicture.Size.Height) / (double)parentH;

			this.lblPosXBR.Text = "" + Math.Round(x * 100, 2) + " %";
			this.lblPosYBR.Text = "" + Math.Round(y * 100, 2) + " %";
		}

        void UpdatePercents(MyPictureBox box)
        {
            if (box.Parent == null)
            {
                return;
            }
            this.textChangedEnabled = false;

			box.WidthPercent = (double)box.Size.Width / (double)box.Parent.Size.Width;
			box.HeightPercent = (double)box.Size.Height / (double)box.Parent.Size.Height;

            box.WidthOffset = 0;
            box.HeightOffset = 0;

            this.txtBoxWidthPercent.Text = "" + Math.Round(box.WidthPercent, 2);
            this.txtBoxHeightPercent.Text = "" + Math.Round(box.HeightPercent, 2);

            this.txtBoxOffsetWidthPx.Text = "" + box.WidthOffset;
            this.txtBoxOffsetHeightPx.Text = "" + box.HeightOffset;

            this.textChangedEnabled = true;
        }

		void CalcPictureBoxSize(MyPictureBox box)
		{
            if (box.Parent == null)
            {
                return;
            }

            this.textChangedEnabled = false;

			double w = box.Parent.Width;
			double h = box.Parent.Height;

            w *= box.WidthPercent;
            h *= box.HeightPercent;

            w += box.WidthOffset;
            h += box.HeightOffset;

            Size s = new Size((int)w, (int)h);
			box.Size = s;

            this.txtBoxWidthPercent.Text = "" + Math.Round(box.WidthPercent, 2);
            this.txtBoxHeightPercent.Text = "" + Math.Round(box.HeightPercent, 2);

            this.txtBoxOffsetWidthPx.Text = "" + box.WidthOffset;
            this.txtBoxOffsetHeightPx.Text = "" + box.HeightOffset;

            this.txtBoxRealWidth.Text = "" + s.Width;
            this.txtBoxRealHeight.Text = "" + s.Height;

            if (box.Image != null)
            {
				double ar = (double)box.Image.Size.Height / (double)box.Image.Size.Width;
                this.lblARInfo.Text = "H / W = " + Math.Round(ar, 2);
            }
            else
            {
                this.lblARInfo.Text = "";
            }

            this.textChangedEnabled = true;

		}

		private void button1_Click(object sender, EventArgs e)
		{
			//change panel basic settings

			this.basicScreen.BackColor = this.pctBoxScreenColor.BackColor;

			int w = -1;
			int h = -1;
			if (int.TryParse(this.textBoxScreenW.Text, out w) == false)
			{
				return;
			}

			if (int.TryParse(this.textBoxScreenH.Text, out h) == false)
			{
				return;
			}

			if (w <= 0)
			{
				return;
			}			
			if (h <= 0)
			{
				return;
			}

			this.basicScreen.Size = new Size(w, h);

			foreach (MyPictureBox p in this.pictures)
			{
				this.CalcPictureBoxSize(p);
			}

			foreach (MyPictureBox p in this.pictures)
			{
				int parentW = p.Parent.Size.Width;
				int parentH = p.Parent.Size.Height;


				Point tmpP = p.Location;

				tmpP.X = (int)(p.TL_X * (double)parentW);
				tmpP.Y = (int)(p.TL_Y * (double)parentH);

				p.Location = tmpP;

				

			}

			

		}

		private void pictureBox1_Click(object sender, EventArgs e)
		{
			DialogResult result = this.colorDialog1.ShowDialog();
			if (result != DialogResult.OK) // Test result.
			{
				return;
			}

			this.pctBoxScreenColor.BackColor = this.colorDialog1.Color;
			
		}

		

		

		
		
	}
}
